const DAOUtils = require('../utils/dao-utils');
const FileUtils = require('../utils/file-utils');
const IssuesDAO = require('../dao/issues-dao');
const fs = require('fs');
const pathLib = require('path');
const axios = require('axios');
const configuration = require('../configuration');

async function downloadImage (url, fileName) {
  
  const path = pathLib.resolve(__dirname, '..', '..', 'tmp', fileName);
  
  // axios image download with response type "stream"
  const response = await axios({
    method: 'GET',
    url,
    headers: { 'Authorization': `Basic ${configuration.TOKEN}` },
    responseType: 'stream'
  });
  
  // pipe the result stream into a file on disc
  response.data.pipe(fs.createWriteStream(path));
  
  // return a promise and resolve when download finishes
  return new Promise((resolve, reject) => {
    response.data.on('end', () => {
      resolve()
    });
    
    response.data.on('error', (err) => {
      reject(err)
    })
  })
  
}

const ISSUE_NAME = 'REF-2436';

function downloadAttachments (issue) {
  issue.fields.attachment.forEach(attachment => downloadImage(attachment.content, `${issue.key}-${attachment.filename}`));
}

async function downloadAttachmentsByKey (issueKey = ISSUE_NAME) {
  const issue = await DAOUtils.fetch(`/issue/${issueKey}`);
  downloadAttachments(issue);
  issue.fields.subtasks.forEach(subtask => downloadAttachmentsByKey(subtask.key));
}

function printIssue (issue, lines) {
  const { key, summary, description } = issue;
  lines.push(`${key}: ${summary}`);
  if (description) {
    lines.push('');
    lines.push(description);
  }
  lines.push('-----------------------------------');
}

async function saveIssueDetails (issueKey = ISSUE_NAME) {
  const issue = await IssuesDAO.findIssueByName(issueKey);
  const lines = [];
  printIssue(issue, lines);
  
  const children = await Promise.all(issue.children.map(child => IssuesDAO.findIssueByName(child.key)));
  children.forEach(subtask => printIssue(subtask, lines));
  FileUtils.saveFile(`${__dirname}/../../tmp/${issueKey}.txt`, lines.join('\n'));
}

(async() => {
  await downloadAttachmentsByKey();
  await saveIssueDetails();
})();
